"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const axis_1 = require("./axis");
class ContinuousAxis extends axis_1.Axis {
    constructor(attrs) {
        super(attrs);
    }
}
exports.ContinuousAxis = ContinuousAxis;
ContinuousAxis.__name__ = "ContinuousAxis";
