"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("./root");
const _a = require("../core/dom");
_a.styles.append(".bk-root .bk-tile-attribution a {\n  color: black;\n}\n");
exports.bk_tile_attribution = "bk-tile-attribution";
