import "es5-ext/object/assign/implement";
import "es5-ext/number/is-integer/implement";
import "es5-ext/string/#/repeat/implement";
import "es5-ext/array/from/implement";
import "es5-ext/math/log10/implement";
import "es6-set/implement";
import "es6-map/implement";
import "es6-weak-map/implement";
import "es6-promise/auto";
