"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const model_1 = require("../../model");
class Ticker extends model_1.Model {
    constructor(attrs) {
        super(attrs);
    }
}
exports.Ticker = Ticker;
Ticker.__name__ = "Ticker";
