"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Tables = require("./index");
exports.Tables = Tables;
const base_1 = require("../../../base");
base_1.register_models(Tables);
