"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const model_1 = require("../../model");
class TickFormatter extends model_1.Model {
    constructor(attrs) {
        super(attrs);
    }
}
exports.TickFormatter = TickFormatter;
TickFormatter.__name__ = "TickFormatter";
